#include <sourcemod>
#include <sdktools_functions>
#include <tf2attributes>

#include <vip_core>

#pragma newdecls required

char g_szFeaturesNames[][] =  {
  "AustraliumWeapons",
  "GoldStatues"
};

// Australium Guns
int g_iAustralium[][] = {
  {
    200, 205, 208, 202,
    201, 45,  228, 206,
    424, 141, 36, 61
  }, {
    16, 203, 20, 207
  }, {
    38, 132, 7, 197,
    194
  }
};

int g_iAustraliumSizes[] = {12, 4, 5};

// Public information
public Plugin myinfo = {
  version   = "2.0",
  author    = "CrazyHackGUT aka Kruzya",
  name      = "[VIP] Modified Weapons",
  url       = "https://g-44.ru/"
};

public APLRes AskPluginLoad2(Handle hMySelf, bool bLate, char[] szErrBuff, int iBuffLength) {
  if (GetEngineVersion() == Engine_TF2)
    return APLRes_Success;

  strcopy(szErrBuff, iBuffLength, "This module works only in Team Fortress 2!");
  return APLRes_Failure;
}

public void OnPluginStart() {
  if (VIP_IsVIPLoaded())
    VIP_OnVIPLoaded();
  
  HookEvent("post_inventory_application", Event_PlayerSpawn,  EventHookMode_Post); // Resupply Locker
  HookEvent("player_changeclass",         Event_PlayerSpawn,  EventHookMode_Post);
}

public void Event_PlayerSpawn(Handle event, const char[] name, bool dontBroadcast) {
  if (!VIP_IsVIPLoaded())
    return;

  int iClient = GetClientOfUserId(GetEventInt(event, "userid"));
  VIP_OnPlayerSpawn(iClient, GetClientTeam(iClient), VIP_IsClientVIP(iClient));
}

public void OnPluginEnd() {
  if(CanTestFeatures() && GetFeatureStatus(FeatureType_Native, "VIP_UnregisterFeature") == FeatureStatus_Available)
    for (int iFeature; iFeature < sizeof(g_szFeaturesNames); iFeature++)
      if (VIP_IsValidFeature(g_szFeaturesNames[iFeature]))
        VIP_UnregisterFeature(g_szFeaturesNames[iFeature]);
}

public void VIP_OnVIPLoaded() {
  for (int iFeature; iFeature < sizeof(g_szFeaturesNames); iFeature++)
    if (!VIP_IsValidFeature(g_szFeaturesNames[iFeature]))
      VIP_RegisterFeature(g_szFeaturesNames[iFeature], BOOL, _, OnClientEnable);
}

public void VIP_OnPlayerSpawn(int iClient, int iTeam, bool bIsVIP) {
  if (!bIsVIP)
    return;

  if (VIP_GetClientFeatureStatus(iClient, g_szFeaturesNames[0]) != NO_ACCESS)
    ChangeAustraliumState(iClient, VIP_IsClientFeatureUse(iClient, g_szFeaturesNames[0]));
  if (VIP_GetClientFeatureStatus(iClient, g_szFeaturesNames[1]) != NO_ACCESS)
    ChangeGoldStatuesState(iClient, VIP_IsClientFeatureUse(iClient, g_szFeaturesNames[1]));
}

public Action OnClientEnable(int iClient, const char[] szFeatureName, VIP_ToggleState EOldStatus, VIP_ToggleState &ENewStatus) {
  bool bState = (ENewStatus == ENABLED) ? true : false;

  if (strcmp(szFeatureName, g_szFeaturesNames[0]) == 0)
    ChangeAustraliumState(iClient, bState);
  else if (strcmp(szFeatureName, g_szFeaturesNames[1]) == 0)
    ChangeGoldStatuesState(iClient, bState);
  
  return Plugin_Changed;
}

/* Australium Weapons */
public void ChangeAustraliumState(int iClient, bool bEnable) {
  if (!IsPlayerAlive(iClient))
    return;

  int iWeapon;
  for (int idx = 0; idx < 3; idx++) {
    iWeapon = GetPlayerWeaponSlot(iClient, idx);
    if (iWeapon > MaxClients && IsValidEntity(iWeapon))
      if (IsItemInArray(GetEntProp(iWeapon, Prop_Send, "m_iItemDefinitionIndex"), g_iAustralium[idx], g_iAustraliumSizes[idx]))
        ChangeAustraliumAttributes(iWeapon, bEnable);
  }
}

public void ChangeAustraliumAttributes(int iWeapon, bool bState) {
  float fValue = (bState) ? 1.0 : 0.0;
  
  SetAttribute(iWeapon, 2027, fValue);
  SetAttribute(iWeapon, 2022, fValue);
  SetAttribute(iWeapon, 542, fValue);
}

/* Gold Statues */
public void ChangeGoldStatuesState(int iClient, bool bEnable) {
  if (!IsPlayerAlive(iClient))
    return;
  
  int iWeapon;
  for (int idx = 0; idx < 3; idx++) {
    iWeapon = GetPlayerWeaponSlot(iClient, idx);
    if (iWeapon > MaxClients && IsValidEntity(iWeapon))
      SetAttribute(iWeapon, 150, ((bEnable) ? 1.0 : 0.0));
  }
}

/* Helpers */
stock bool IsItemInArray(int iItem, int[] iArray, int iArrayLength) {
  for (int iCurrentItem = 0; iCurrentItem < iArrayLength; iCurrentItem++)
    if (iArray[iCurrentItem] == iItem)
      return true;
  return false;
}

stock void SetAttribute(int iWeapon, int iAttrib, float fValue) {
  TF2Attrib_SetByDefIndex(iWeapon, iAttrib, fValue);
  
  if (fValue == 0.0)
    TF2Attrib_RemoveByDefIndex(iWeapon, iAttrib);
}
